#!/usr/bin/env python3

from argparse import ArgumentParser
from itertools import combinations
from sys import stdout
import networkx as nx
import numpy as np


def readMatrix(data):

    res = list()
    for line in data:
        res.append(list(map(int, line[:-1])))
    return np.array(res, dtype=bool)


def constructIncompatiblityMatrix(M):
    """ constructs only upper triangle matrix """

    res = np.zeros(shape = (M.shape[1], M.shape[1]), dtype=bool)
    for i, j in combinations(range(M.shape[1]), 2):
        types = set((M[k, i], M[k, j]) for k in range(M.shape[0]) if M[k,i] or
                M[k, j])
        res[i, j] = len(types) == 3
    return res


def writeILP(G, out):

    # objective 
    print('maximize {}'.format(' + '.join('x{}'.format(v) for v in G.nodes())),
            file=out)

    # constraints
    print('subject to', file=out)
    # any pair of nodes sharing an edge cannot simultaneously be part of the
    # same solution
    for u, v in G.edges():
        print('x{} + x{} <= 1'.format(u, v), file=out)

    # declaration of binary variables
    print('binary', file=out)
    for v in G.nodes():
        print('x{}'.format(v), file=out)
    print('end', file=out)


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('binary_matrix', type=open, help='zero/one matrix')
    args = parser.parse_args()

    # read character matrix
    M = readMatrix(args.binary_matrix)
    # character incompatibility matrix
    I = constructIncompatiblityMatrix(M)
    # graph with vertices representing characters; any two vertices share an
    # edge if their corresponding characters are incompatible
    G = nx.from_numpy_matrix(I)
    # construct & output ILP
    writeILP(G, stdout)
